==> tlmpwork-0.1.src.tar.gz: tlmpwork-0.1/graph/grnode.cc <==
#include <string.h>
#include <dialog.h>
#include <popen.h>
#include <proto.h>
#include "graph.h"

FILE *fpov = NULL;
int fy;

PUBLIC GR_NODE::GR_NODE()
{
	x = y = 0;
	width = height = depth = 0;
	level_right = 0;
	level_sub = 0;
	memset (tbright,0,sizeof(tbright));
	memset (tbsub,0,sizeof(tbsub));
	nbsub = nbright = 0;
	right.x = right.y = 0;
	left.x = left.y = 0;
	top.x = top.y = 0;
	bottom.x = bottom.y = 0;
	margin.top = margin.bottom = 0;
	margin.left = margin.right = 0;
}

PUBLIC GR_NODE::~GR_NODE()
{
	for (int i=0; i<nbsub; i++) delete tbsub[i];
	for (int i=0; i<nbright; i++) delete tbright[i];
}

void _F_grnode::right1(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right2(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right3(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right4(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right5(GRAPH_MODE mode, GR_NODE &node)
{
}
int _F_grnode::sub1(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub2(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub3(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub4(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub5(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}

PUBLIC void GR_BASE::drawrect (
	GRAPH_MODE mode,
	GR_NODE &node,
	int w,
	int h,
	const char *title,		// Message on top of the circle
	const char *desc)		// Message inside
{
	::drawrect (mode,node,w,h,title,desc);
}
/*
	Dessine un rectangle
*/
PUBLIC void GR_BASE::drawrect (GRAPH_MODE mode, GR_NODE &node, int w, int h)
{
	drawrect (mode,node,w,h,"","");
}


PUBLIC void GR_BASE::drawellipse (
	GRAPH_MODE mode,
	GR_NODE &node,
	int w,
	int h,
	const char *title,		// Message on top of the circle
	const char *desc)		// Message inside
{
	::drawellipse (mode,node,w,h,title,desc);
}

PUBLIC void GR_BASE::drawellipse (GRAPH_MODE mode, GR_NODE &node, int w, int h)
{
	drawellipse (mode,node,w,h,"","");
}

void grnode_getcoor (GR_NODE &node, ANCHOR anch, int &x, int &y)
{
	if (anch == ANCHOR_LEFT){
		x = node.left.x;
		y = node.left.y;
	}else if (anch == ANCHOR_TOP){
		x = node.top.x;
		y = node.top.y;
	}else if (anch == ANCHOR_RIGHT){
		x = node.right.x;
		y = node.right.y;
	}else if (anch == ANCHOR_BOTTOM){
		x = node.bottom.x;
		y = node.bottom.y;
	}
}

PUBLIC void GR_BASE::linkline (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	int x1,y1;
	grnode_getcoor (node1,anch1,x1,y1);
	int x2,y2;
	grnode_getcoor (node2,anch2,x2,y2);
	drawline (x1,y1,x2,y2,2);
}

PUBLIC void GR_BASE::linkarrow (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	int x1,y1;
	grnode_getcoor (node1,anch1,x1,y1);
	int x2,y2;
	grnode_getcoor (node2,anch2,x2,y2);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x1,y1,x2,y2);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x2-5,y2-5,x2+5,y2+5);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x2+5,y2-5,x2-5,y2+5);
}

PUBLIC void GR_BASE::linkarcarrow (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	linkarrow (node1,anch1,node2,anch2);
}


void _F_grnode::link1(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link2(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link3(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link4(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link5(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}

int grnode (_F_grnode &c, GRAPH_MODE mode, GR_NODE &node)
{
	node.width = node.height = node.depth = 0;
	node.margin.top = node.margin.right = node.margin.left = 0;
	node.margin.bottom = 0;
	int depth = 0;
	int height = 0;
	int width = 0;
	for (int e=0; e<2; e++){
		if (e == 1 && mode != GR_DRAW) break;
		GRAPH_MODE emode = e == 0 ? GR_EVAL : GR_DRAW;
		if (e == 0) c.draw (emode,node);
		width = node.margin.top + 5;
		GR_NODE tbright[5];
		for (int i=0; i<5; i++){
			GR_NODE right,sub;
			int ret_sub = -2;
			GR_NODE *ptn = tbright + i;
			ptn->y = node.y + width;
			ptn->x = node.x + node.depth+20;
			switch (i){
			case 0:
				c.right1(emode,*ptn);
				ret_sub = c.sub1(emode,sub);
				break;
			case 1:
				c.right2(emode,*ptn);
				ret_sub = c.sub2(emode,sub);
				break;
			case 2:
				c.right3(emode,*ptn);
				ret_sub = c.sub3(emode,sub);
				break;
			case 3:
				c.right4(emode,*ptn);
				ret_sub = c.sub4(emode,sub);
				break;
			case 4:
				c.right5(emode,*ptn);
				ret_sub = c.sub5(emode,sub);
				break;
			}
			if (ptn->width > 0){
				width += ptn->width + ptn->margin.top + ptn->margin.bottom + 5;
				if (depth < ptn->depth) depth = ptn->depth;
				if (height < ptn->height) height = ptn->height;
			}
			if (ret_sub == 0){
				width += sub.width+10;
				if (depth < sub.depth) depth = sub.depth;
				if (height < sub.height) height = sub.height;
			}
		}
		if (e == 1){
			int y = node.y;
			if (width == 0) width = node.width;
			node.y = y + width/2 - node.width/2;
			c.draw (emode,node);
			for (int i=0; i<5; i++){
				GR_NODE *ptn = tbright + i;
				if (ptn->width != 0){
					switch (i){
					case 0:
						c.link1 (node,*ptn);
						break;
					case 1:
						c.link2 (node,*ptn);
						break;
					case 2:
						c.link3 (node,*ptn);
						break;
					case 3:
						c.link4 (node,*ptn);
						break;
					case 4:
						c.link5 (node,*ptn);
						break;
					}
				}
			}
			node.y = y;
		}
	}
	if (node.width < width) node.width = width;
	node.depth += depth+20;
	if (node.height < height) node.height = height;
	return 0;
}	

int grgroup (_F_grgroup &c, GRAPH_MODE mode, GR_NODE &node)
{
	node.width = node.height = node.depth = 0;
	node.margin.top = node.margin.right = node.margin.left = 0;
	node.margin.bottom = 0;
	{
		GR_NODE gnode;
		c.group (GR_EVAL,gnode);
		node.width = gnode.width + 10;
		node.depth = gnode.depth + 10;
		c.draw (GR_EVAL,node);
	}
	if (mode == GR_DRAW){
		GR_NODE gnode;
		gnode.x = node.x + 5;
		gnode.y = node.y + node.margin.top + 5;
		c.group (GR_DRAW,gnode);
		c.draw  (GR_DRAW,node);
	}
	// node.width += node.margin.top + node.margin.bottom;
	return 0;
}	

PUBLIC UNIFORM_OBJECT::UNIFORM_OBJECT(_F_UNIFORM_OBJECT &_c)
{
	maxwidth = maxdepth = 20;
	maxheight = 0;
	c = &_c;
}

PRIVATE void UNIFORM_OBJECT::fitsize (GR_NODE &node)
{
	if (maxwidth > node.width){
		node.width = maxwidth;
	}else{
		maxwidth = node.width;
	}
	if (maxheight > node.height){
		node.height = maxheight;
	}else{
		maxheight = node.height;
	}
	if (maxdepth > node.depth){
		node.depth = maxdepth;
	}else{
		maxdepth = node.depth;
	}
}

PUBLIC void UNIFORM_OBJECT::draw (
	GRAPH_MODE mode,
	GR_NODE &node,
	const char *desc)
{
	draw (mode,node,"",desc);
}

PUBLIC void UNIFORM_OBJECT::draw (
	GRAPH_MODE mode,
	GR_NODE &node,
	const char *title,
	const char *desc)
{
	fitsize (node);
	c->draw (mode,node,maxdepth,maxwidth,title,desc);
	fitsize (node);
}


int grmanage (_F_grmanage &c, const char *title, const char *png)
{
	dialog_clear();
	GR_NODE node;
	c.exec (GR_EVAL,node);
	char mainid[10];
	sprintf (mainid,"main-%d",uithread_id);
	char tmp[1000];
	diagui_sendcmd (P_MainForm,"%s %s\n",mainid,diagui_quote(title,tmp));
//	if (node.width > 1000){
//		node.width = 1000;
//	}
//	if (node.depth > 700){
//		node.depth = 700;
//	}
	diagui_sendcmd (P_Form,"graph $w=%d h=%d htrigger=1000 vtrigger=700\n",node.depth+10,node.width);
	fpov = fopen ("/tmp/test.ini","w");
	fy = node.width;
	fprintf (fpov,"#include \"colors.inc\"\n");
	fprintf (fpov,"#include \"stones.inc\"\n");
	fprintf (fpov,"#include \"textures.inc\"\n");
	fprintf (fpov,"#include \"shapes.inc\"\n");
	fprintf (fpov,"#include \"glass.inc\"\n");
	fprintf (fpov,"#include \"metals.inc\"\n");
	fprintf (fpov,"#include \"woods.inc\"\n");
	fprintf (fpov,"background { color Cyan }\n");
	fprintf (fpov,"camera {\n");
	fprintf (fpov,"\tlocation <%d, %d, %d>\n",node.depth/2,node.width/2,-node.depth);
	fprintf (fpov,"\tlook_at  <%d, %d,  0>\n",node.depth/2,node.width/2);
	fprintf (fpov," }\n");
	c.exec (GR_DRAW,node);
	fprintf (fpov,"light_source { <%d, %d, %d> color White}\n",node.width/2,node.depth,-node.depth);
	fclose (fpov);
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_End,"\n");
	if (png == NULL){
		SSTRING s1,s2,s3;
		diagui_sync (mainid,s1,s2,s3);
	}else{
		SSTRING cmd;
		cmd.setfromf ("/usr/local/bin/x-povray +A -I/tmp/test.ini -O/tmp/%s.png -W800 -H600",png);
		POPEN pop (cmd.get());
		if (pop.isok()){
			while (pop.wait(1)>=0){
				char line[1000];
				while (pop.readout(line,sizeof(line)-1)!=-1){
					printf ("%s\n",line);
				}
				while (pop.readerr(line,sizeof(line)-1)!=-1){
					fprintf (stderr,"%s\n",line);
				}
			}
		}else{
			xconf_error ("Pas ok: %s",cmd.get());
		}
	}
	return 0;
}

int grmanage (_F_grmanage &c, const char *title)
{
	grmanage (c,title,NULL);
	return 0;
}

==> tlmpwork-0.5.src.tar.gz: tlmpwork-0.5/graph/grnode.cc <==
#include <string.h>
#include <dialog.h>
#include <popen.h>
#include <proto.h>
#include "graph.h"

FILE *fpov = NULL;
int fy;

PUBLIC GR_NODE::GR_NODE()
{
	x = y = 0;
	width = height = depth = 0;
	level_right = 0;
	level_sub = 0;
	memset (tbright,0,sizeof(tbright));
	memset (tbsub,0,sizeof(tbsub));
	nbsub = nbright = 0;
	right.x = right.y = 0;
	left.x = left.y = 0;
	top.x = top.y = 0;
	bottom.x = bottom.y = 0;
	margin.top = margin.bottom = 0;
	margin.left = margin.right = 0;
}

PUBLIC GR_NODE::~GR_NODE()
{
	for (int i=0; i<nbsub; i++) delete tbsub[i];
	for (int i=0; i<nbright; i++) delete tbright[i];
}

void _F_grnode::right1(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right2(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right3(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right4(GRAPH_MODE mode, GR_NODE &node)
{
}
void _F_grnode::right5(GRAPH_MODE mode, GR_NODE &node)
{
}
int _F_grnode::sub1(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub2(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub3(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub4(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}
int _F_grnode::sub5(GRAPH_MODE mode, GR_NODE &node)
{
	return -2;
}

PUBLIC void GR_BASE::drawrect (
	GRAPH_MODE mode,
	GR_NODE &node,
	int w,
	int h,
	const char *title,		// Message on top of the circle
	const char *desc)		// Message inside
{
	::drawrect (mode,node,w,h,title,desc);
}
/*
	Dessine un rectangle
*/
PUBLIC void GR_BASE::drawrect (GRAPH_MODE mode, GR_NODE &node, int w, int h)
{
	drawrect (mode,node,w,h,"","");
}


PUBLIC void GR_BASE::drawellipse (
	GRAPH_MODE mode,
	GR_NODE &node,
	int w,
	int h,
	const char *title,		// Message on top of the circle
	const char *desc)		// Message inside
{
	::drawellipse (mode,node,w,h,title,desc);
}

PUBLIC void GR_BASE::drawellipse (GRAPH_MODE mode, GR_NODE &node, int w, int h)
{
	drawellipse (mode,node,w,h,"","");
}

void grnode_getcoor (GR_NODE &node, ANCHOR anch, int &x, int &y)
{
	if (anch == ANCHOR_LEFT){
		x = node.left.x;
		y = node.left.y;
	}else if (anch == ANCHOR_TOP){
		x = node.top.x;
		y = node.top.y;
	}else if (anch == ANCHOR_RIGHT){
		x = node.right.x;
		y = node.right.y;
	}else if (anch == ANCHOR_BOTTOM){
		x = node.bottom.x;
		y = node.bottom.y;
	}
}

PUBLIC void GR_BASE::linkline (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	int x1,y1;
	grnode_getcoor (node1,anch1,x1,y1);
	int x2,y2;
	grnode_getcoor (node2,anch2,x2,y2);
	drawline (x1,y1,x2,y2,2);
}

PUBLIC void GR_BASE::linkarrow (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	int x1,y1;
	grnode_getcoor (node1,anch1,x1,y1);
	int x2,y2;
	grnode_getcoor (node2,anch2,x2,y2);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x1,y1,x2,y2);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x2-5,y2-5,x2+5,y2+5);
	diagui_sendcmd (P_Drawline,"%d %d %d %d\n",x2+5,y2-5,x2-5,y2+5);
}

PUBLIC void GR_BASE::linkarcarrow (
	GR_NODE &node1,
	ANCHOR anch1,
	GR_NODE &node2,
	ANCHOR anch2)
{
	linkarrow (node1,anch1,node2,anch2);
}


void _F_grnode::link1(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link2(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link3(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link4(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}
void _F_grnode::link5(GR_NODE &node1, GR_NODE &node2)
{
	linkline (node1,ANCHOR_RIGHT,node2,ANCHOR_LEFT);
}

void grnode (_F_grnode &c, GRAPH_MODE mode, GR_NODE &node)
{
	node.width = node.height = node.depth = 0;
	node.margin.top = node.margin.right = node.margin.left = 0;
	node.margin.bottom = 0;
	int depth = 0;
	int height = 0;
	int width = 0;
	for (int e=0; e<2; e++){
		if (e == 1 && mode != GR_DRAW) break;
		GRAPH_MODE emode = e == 0 ? GR_EVAL : GR_DRAW;
		if (e == 0) c.draw (emode,node);
		width = node.margin.top + 5;
		GR_NODE tbright[5];
		for (int i=0; i<5; i++){
			GR_NODE right,sub;
			int ret_sub = -2;
			GR_NODE *ptn = tbright + i;
			ptn->y = node.y + width;
			ptn->x = node.x + node.depth+20;
			switch (i){
			case 0:
				c.right1(emode,*ptn);
				ret_sub = c.sub1(emode,sub);
				break;
			case 1:
				c.right2(emode,*ptn);
				ret_sub = c.sub2(emode,sub);
				break;
			case 2:
				c.right3(emode,*ptn);
				ret_sub = c.sub3(emode,sub);
				break;
			case 3:
				c.right4(emode,*ptn);
				ret_sub = c.sub4(emode,sub);
				break;
			case 4:
				c.right5(emode,*ptn);
				ret_sub = c.sub5(emode,sub);
				break;
			}
			if (ptn->width > 0){
				width += ptn->width + ptn->margin.top + ptn->margin.bottom + 5;
				if (depth < ptn->depth) depth = ptn->depth;
				if (height < ptn->height) height = ptn->height;
			}
			if (ret_sub == 0){
				width += sub.width+10;
				if (depth < sub.depth) depth = sub.depth;
				if (height < sub.height) height = sub.height;
			}
		}
		if (e == 1){
			int y = node.y;
			if (width == 0) width = node.width;
			node.y = y + width/2 - node.width/2;
			c.draw (emode,node);
			for (int i=0; i<5; i++){
				GR_NODE *ptn = tbright + i;
				if (ptn->width != 0){
					switch (i){
					case 0:
						c.link1 (node,*ptn);
						break;
					case 1:
						c.link2 (node,*ptn);
						break;
					case 2:
						c.link3 (node,*ptn);
						break;
					case 3:
						c.link4 (node,*ptn);
						break;
					case 4:
						c.link5 (node,*ptn);
						break;
					}
				}
			}
			node.y = y;
		}
	}
	if (node.width < width) node.width = width;
	node.depth += depth+20;
	if (node.height < height) node.height = height;
}	

void grgroup (_F_grgroup &c, GRAPH_MODE mode, GR_NODE &node)
{
	node.width = node.height = node.depth = 0;
	node.margin.top = node.margin.right = node.margin.left = 0;
	node.margin.bottom = 0;
	{
		GR_NODE gnode;
		c.group (GR_EVAL,gnode);
		node.width = gnode.width + 10;
		node.depth = gnode.depth + 10;
		c.draw (GR_EVAL,node);
	}
	if (mode == GR_DRAW){
		GR_NODE gnode;
		gnode.x = node.x + 5;
		gnode.y = node.y + node.margin.top + 5;
		c.group (GR_DRAW,gnode);
		c.draw  (GR_DRAW,node);
	}
	// node.width += node.margin.top + node.margin.bottom;
}	

PUBLIC UNIFORM_OBJECT::UNIFORM_OBJECT(_F_UNIFORM_OBJECT &_c)
{
	maxwidth = maxdepth = 20;
	maxheight = 0;
	c = &_c;
}

PRIVATE void UNIFORM_OBJECT::fitsize (GR_NODE &node)
{
	if (maxwidth > node.width){
		node.width = maxwidth;
	}else{
		maxwidth = node.width;
	}
	if (maxheight > node.height){
		node.height = maxheight;
	}else{
		maxheight = node.height;
	}
	if (maxdepth > node.depth){
		node.depth = maxdepth;
	}else{
		maxdepth = node.depth;
	}
}

PUBLIC void UNIFORM_OBJECT::draw (
	GRAPH_MODE mode,
	GR_NODE &node,
	const char *desc)
{
	draw (mode,node,"",desc);
}

PUBLIC void UNIFORM_OBJECT::draw (
	GRAPH_MODE mode,
	GR_NODE &node,
	const char *title,
	const char *desc)
{
	fitsize (node);
	c->draw (mode,node,maxdepth,maxwidth,title,desc);
	fitsize (node);
}


int grmanage (_F_grmanage &c, const char *title, const char *png)
{
	dialog_clear();
	GR_NODE node;
	c.exec (GR_EVAL,node);
	char mainid[10];
	sprintf (mainid,"main-%d",uithread_id);
	char tmp[1000];
	diagui_sendcmd (P_MainForm,"%s %s\n",mainid,diagui_quote(title,tmp));
//	if (node.width > 1000){
//		node.width = 1000;
//	}
//	if (node.depth > 700){
//		node.depth = 700;
//	}
	diagui_sendcmd (P_Form,"graph $w=%d h=%d htrigger=1000 vtrigger=700\n",node.depth+10,node.width);
	fpov = fopen ("/tmp/test.ini","w");
	fy = node.width;
	fprintf (fpov,"#include \"colors.inc\"\n");
	fprintf (fpov,"#include \"stones.inc\"\n");
	fprintf (fpov,"#include \"textures.inc\"\n");
	fprintf (fpov,"#include \"shapes.inc\"\n");
	fprintf (fpov,"#include \"glass.inc\"\n");
	fprintf (fpov,"#include \"metals.inc\"\n");
	fprintf (fpov,"#include \"woods.inc\"\n");
	fprintf (fpov,"background { color Cyan }\n");
	fprintf (fpov,"camera {\n");
	fprintf (fpov,"\tlocation <%d, %d, %d>\n",node.depth/2,node.width/2,-node.depth);
	fprintf (fpov,"\tlook_at  <%d, %d,  0>\n",node.depth/2,node.width/2);
	fprintf (fpov," }\n");
	c.exec (GR_DRAW,node);
	fprintf (fpov,"light_source { <%d, %d, %d> color White}\n",node.width/2,node.depth,-node.depth);
	fclose (fpov);
	diagui_sendcmd (P_End,"\n");
	diagui_sendcmd (P_End,"\n");
	if (png == NULL){
		SSTRING s1,s2,s3;
		diagui_sync (mainid,s1,s2,s3);
	}else{
		SSTRING cmd;
		cmd.setfromf ("/usr/local/bin/x-povray +A -I/tmp/test.ini -O/tmp/%s.png -W800 -H600",png);
		POPEN pop (cmd.get());
		if (pop.isok()){
			while (pop.wait(1)>=0){
				char line[1000];
				while (pop.readout(line,sizeof(line)-1)!=-1){
					printf ("%s\n",line);
				}
				while (pop.readerr(line,sizeof(line)-1)!=-1){
					fprintf (stderr,"%s\n",line);
				}
			}
		}else{
			xconf_error ("Pas ok: %s",cmd.get());
		}
	}
	return 0;
}

int grmanage (_F_grmanage &c, const char *title)
{
	grmanage (c,title,NULL);
	return 0;
}
